@@ -1,7 +1,7 @@
 /*
  * Goodvibes Radio Player
  *
- * Copyright (C) 2015-2018 Arnaud Rebillout
+ * Copyright (C) 2015-2020 Arnaud Rebillout
  *
  * SPDX-License-Identifier: GPL-3.0-or-later
  *
@@ -44,31 +44,31 @@
         "  <uri>https://direct.fipradio.fr/live/fip-midfi.mp3</uri>" \
         "</Station>" \
         "<Station>" \
-        "  <name>FIP Autour du Rock</name>" \
+        "  <name>FIP Rock</name>" \
         "  <uri>https://direct.fipradio.fr/live/fip-webradio1.mp3</uri>" \
         "</Station>" \
         "<Station>" \
-        "  <name>FIP Autour du Jazz</name>" \
+        "  <name>FIP Jazz</name>" \
         "  <uri>https://direct.fipradio.fr/live/fip-webradio2.mp3</uri>" \
         "</Station>" \
         "<Station>" \
-        "  <name>FIP Autour du Groove</name>" \
+        "  <name>FIP Groove</name>" \
         "  <uri>https://direct.fipradio.fr/live/fip-webradio3.mp3</uri>" \
         "</Station>" \
         "<Station>" \
-        "  <name>FIP Autour du Monde</name>" \
+        "  <name>FIP Monde</name>" \
         "  <uri>https://direct.fipradio.fr/live/fip-webradio4.mp3</uri>" \
         "</Station>" \
         "<Station>" \
-        "  <name>FIP Tout nouveau, tout FIP</name>" \
+        "  <name>FIP Nouveautés</name>" \
         "  <uri>https://direct.fipradio.fr/live/fip-webradio5.mp3</uri>" \
         "</Station>" \
         "<Station>" \
-        "  <name>FIP Autour du Reggae</name>" \
+        "  <name>FIP Reggae</name>" \
         "  <uri>https://direct.fipradio.fr/live/fip-webradio6.mp3</uri>" \
         "</Station>" \
         "<Station>" \
-        "  <name>FIP Autour de l'Électro</name>" \
+        "  <name>FIP Electro</name>" \
         "  <uri>https://direct.fipradio.fr/live/fip-webradio8.mp3</uri>" \
         "</Station>"
 
@@ -89,10 +89,6 @@
         "<Station>" \
         "  <name>Nova la Nuit</name>" \
         "  <uri>http://nova-ln.ice.infomaniak.ch/nova-ln-128</uri>" \
-        "</Station>" \
-        "<Station>" \
-        "  <name>Nova V.F.</name>" \
-        "  <uri>http://nova-vf.ice.infomaniak.ch/nova-vf-128</uri>" \
         "</Station>"
 
 /*
@@ -182,6 +178,22 @@
 #define SAVE_DELAY 1 // how long to wait before writing changes to disk
 #define STATION_LIST_FILE "stations.xml" // where to write the stations
 
+/*
+ * Properties
+ */
+
+enum {
+	/* Reserved */
+	PROP_0,
+	/* Properties */
+	PROP_LOAD_PATH,
+	PROP_SAVE_PATH,
+	/* Number of properties */
+	PROP_N,
+};
+
+static GParamSpec *properties[PROP_N];
+
 /*
  * Signals
  */
@@ -203,11 +215,15 @@ static guint signals[SIGNAL_N];
  */
 
 struct _GvStationListPrivate {
-	/* Load/save paths */
-	gchar **load_paths;
+	/* Paths */
+	gchar **default_load_paths;
+	gchar  *default_save_path;
+	gchar  *load_path;
 	gchar  *save_path;
 	/* Timeout id, > 0 if a save operation is scheduled */
 	guint   save_source_id;
+	/* Set to true during object finalization */
+	gboolean finalization;
 	/* Ordered list of stations */
 	GList  *stations;
 	/* Shuffled list of stations, automatically created
@@ -380,8 +396,8 @@ markup_on_error(GMarkupParseContext *context G_GNUC_UNUSED,
 	parsing->user_agent = NULL;
 }
 
-static GList *
-parse_markup(const gchar *text, GError **err)
+static gboolean
+parse_markup(const gchar *text, GList **list, GError **err)
 {
 	GMarkupParseContext *context;
 	GMarkupParser parser = {
@@ -398,12 +414,22 @@ parse_markup(const gchar *text, GError **err)
 		NULL,
 		NULL
 	};
+	gboolean ret;
+
+	g_return_val_if_fail(list != NULL, FALSE);
+	g_return_val_if_fail(err == NULL || *err == NULL, FALSE);
 
 	context = g_markup_parse_context_new(&parser, 0, &parsing, NULL);
-	g_markup_parse_context_parse(context, text, -1, err);
+	ret = g_markup_parse_context_parse(context, text, -1, err);
 	g_markup_parse_context_free(context);
 
-	return g_list_reverse(parsing.list);
+	if (ret == FALSE) {
+		g_assert(err == NULL || *err != NULL);
+		return FALSE;
+	}
+
+	*list = g_list_reverse(parsing.list);
+	return TRUE;
 }
 
 static GString *
@@ -428,7 +454,7 @@ print_markup_station(GvStation *station)
 
 	/* A station is supposed to have an uri */
 	if (uri == NULL) {
-		WARNING("Station (%s) has no uri !", name);
+		WARNING("Station (%s) has no uri!", name);
 		return NULL;
 	}
 
@@ -450,12 +476,16 @@ print_markup_station(GvStation *station)
 	return g_string_free(string, FALSE);
 }
 
-static gchar *
-print_markup(GList *list, GError **err G_GNUC_UNUSED)
+static gboolean
+print_markup(GList *list, gchar **markup, GError **err)
 {
 	GList *item;
-	GString *string = g_string_new(NULL);
+	GString *string;
 
+	g_return_val_if_fail(markup != NULL, FALSE);
+	g_return_val_if_fail(err == NULL || *err == NULL, FALSE);
+
+	string = g_string_new(NULL);
 	g_string_append(string, "<Stations>\n");
 
 	for (item = list; item; item = item->next) {
@@ -471,8 +501,134 @@ print_markup(GList *list, GError **err G_GNUC_UNUSED)
 	}
 
 	g_string_append(string, "</Stations>");
+	*markup = g_string_free(string, FALSE);
 
-	return g_string_free(string, FALSE);
+	return TRUE;
+}
+
+/*
+ * File I/O
+ */
+
+static gboolean
+load_station_list_from_string(const gchar *text, GList **list, GError **error)
+{
+	return parse_markup(text, list, error);
+}
+
+static gboolean
+load_station_list_from_file(const gchar *path, GList **list, GError **error)
+{
+	gchar *text = NULL;
+	gboolean ret;
+
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+
+	ret = g_file_get_contents(path, &text, NULL, error);
+	if (ret == FALSE) {
+		g_assert (error == NULL || *error != NULL);
+		goto end;
+	}
+
+	ret = load_station_list_from_string(text, list, error);
+	if (ret == FALSE) {
+		g_assert(error == NULL || *error != NULL);
+		goto end;
+	}
+
+end:
+	g_free(text);
+	return ret;
+}
+
+static gboolean
+save_station_list_to_string(GList *list, gchar **text, GError **error)
+{
+	return print_markup(list, text, error);
+}
+
+static gboolean
+save_station_list_to_file(GList *list, const gchar *path, GError **error)
+{
+	gboolean ret;
+	gchar *text = NULL;
+	gchar *dirname = NULL;
+
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+
+	ret = save_station_list_to_string(list, &text, error);
+	if (ret == FALSE) {
+		g_assert(error == NULL || *error != NULL);
+		goto end;
+	}
+
+	/* g_file_set_contents() doesn't work with /dev/null, due to
+	 * the way it's implemented (see documentation for details).
+	 * However it's convenient to set the save path to /dev/null
+	 * when we actually don't want to save (ie. test suite).
+	 * So we handle this special case explicitly here.
+	 */
+	if (!g_strcmp0(path, "/dev/null")) {
+		ret = TRUE;
+		goto end;
+	}
+
+	dirname = g_path_get_dirname(path);
+	if (g_mkdir_with_parents(dirname, S_IRWXU) != 0) {
+		g_set_error(error, G_FILE_ERROR,
+		            g_file_error_from_errno(errno),
+		            "Failed to make directory: %s", g_strerror(errno));
+		ret = FALSE;
+		goto end;
+	}
+
+	ret = g_file_set_contents(path, text, -1, error);
+	if (ret == FALSE) {
+		g_assert(error == NULL || *error != NULL);
+		goto end;
+	}
+
+end:
+	g_free(dirname);
+	g_free(text);
+	return ret;
+}
+
+static void
+relocate_station_list_file_for_4_1(const gchar *new_file)
+{
+	const gchar *user_config_dir;
+	gchar *old_file = NULL;
+	gchar *dirname = NULL;
+
+	user_config_dir = gv_get_app_user_config_dir();
+	old_file = g_build_filename(user_config_dir, "stations", NULL);
+
+	if (!g_file_test(old_file, G_FILE_TEST_EXISTS))
+		goto cleanup;
+
+	INFO("Station list migration: '%s' > '%s'", old_file, new_file);
+
+	dirname = g_path_get_dirname(new_file);
+	if (g_mkdir_with_parents(dirname, S_IRWXU) != 0) {
+		WARNING("Failed to make directory '%s': %s",
+			dirname, strerror(errno));
+		goto cleanup;
+	}
+	g_free(dirname);
+
+	if (g_rename(old_file, new_file) != 0) {
+		WARNING("Failed to rename file '%s' to '%s': %s",
+			old_file, new_file, strerror(errno));
+		goto cleanup;
+	}
+
+	dirname = g_path_get_dirname(old_file);
+	g_rmdir(dirname);
+
+ cleanup:
+	g_free(dirname);
+	g_free(old_file);
 }
 
 /*
@@ -653,6 +809,94 @@ on_station_notify(GvStation     *station,
 	g_signal_emit(self, signals[SIGNAL_STATION_MODIFIED], 0, station);
 }
 
+/*
+ * Property accessors
+ */
+
+const gchar *
+gv_station_list_get_load_path(GvStationList *self)
+{
+	return self->priv->load_path;
+}
+
+static void
+gv_station_list_set_load_path(GvStationList *self, const gchar *path)
+{
+	GvStationListPrivate *priv = self->priv;
+
+        if (!g_strcmp0(priv->load_path, path))
+                return;
+
+        g_free(priv->load_path);
+        priv->load_path = g_strdup(path);
+        g_object_notify_by_pspec(G_OBJECT(self), properties[PROP_LOAD_PATH]);
+}
+
+const gchar *
+gv_station_list_get_save_path(GvStationList *self)
+{
+	return self->priv->save_path;
+}
+
+static void
+gv_station_list_set_save_path(GvStationList *self, const gchar *path)
+{
+	GvStationListPrivate *priv = self->priv;
+
+        if (!g_strcmp0(priv->save_path, path))
+                return;
+
+        g_free(priv->save_path);
+        priv->save_path = g_strdup(path);
+        g_object_notify_by_pspec(G_OBJECT(self), properties[PROP_SAVE_PATH]);
+}
+
+static void
+gv_station_list_get_property(GObject    *object,
+                             guint       property_id,
+                             GValue     *value G_GNUC_UNUSED,
+                             GParamSpec *pspec)
+{
+	GvStationList *self = GV_STATION_LIST(object);
+
+	TRACE_GET_PROPERTY(object, property_id, value, pspec);
+
+	switch (property_id) {
+	case PROP_LOAD_PATH:
+		g_value_set_string(value, gv_station_list_get_load_path(self));
+		break;
+	case PROP_SAVE_PATH:
+		g_value_set_string(value, gv_station_list_get_save_path(self));
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
+		break;
+	}
+}
+
+static void
+gv_station_list_set_property(GObject      *object,
+                             guint         property_id,
+                             const GValue *value,
+                             GParamSpec   *pspec)
+{
+	GvStationList *self = GV_STATION_LIST(object);
+
+	TRACE_SET_PROPERTY(object, property_id, value, pspec);
+
+	switch (property_id) {
+	case PROP_LOAD_PATH:
+		gv_station_list_set_load_path(self, g_value_get_string(value));
+		break;
+	case PROP_SAVE_PATH:
+		gv_station_list_set_save_path(self, g_value_get_string(value));
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
+		break;
+	}
+}
+
 /*
  * Public functions
  */
@@ -712,11 +956,7 @@ gv_station_list_insert(GvStationList *self, GvStation *station, gint pos)
 	GvStationListPrivate *priv = self->priv;
 	GList *similar_item;
 
-	/* Ensure a valid station was given */
-	if (station == NULL) {
-		WARNING("Attempting to insert NULL station");
-		return;
-	}
+	g_return_if_fail(station != NULL);
 
 	/* Give info */
 	INFO("Inserting station '%s'", gv_station_get_name_or_uri(station));
@@ -755,43 +995,45 @@ gv_station_list_insert(GvStationList *self, GvStation *station, gint pos)
 	gv_station_list_save_delayed(self);
 }
 
-/* Insert a station before another.
- * If 'before' is NULL or is not found, the station is appended at the end of the list.
- */
 void
 gv_station_list_insert_before(GvStationList *self, GvStation *station, GvStation *before)
 {
 	GvStationListPrivate *priv = self->priv;
-	gint pos = -1;
+	gint pos;
+
+	g_return_if_fail(before != NULL);
 
 	pos = g_list_index(priv->stations, before);
+	g_return_if_fail(pos != -1);
+
 	gv_station_list_insert(self, station, pos);
 }
 
-/* Insert a station after another.
- * If 'after' is NULL or not found, the station is appended at the beginning of the list.
- */
 void
 gv_station_list_insert_after(GvStationList *self, GvStation *station, GvStation *after)
 {
 	GvStationListPrivate *priv = self->priv;
-	gint pos = 0;
+	gint pos;
+
+	g_return_if_fail(after != NULL);
 
 	pos = g_list_index(priv->stations, after);
-	pos += 1; /* tricky but does what we want even for pos == -1 */
+	g_return_if_fail(pos != -1);
+
+	pos += 1;
 	gv_station_list_insert(self, station, pos);
 }
 
 void
 gv_station_list_prepend(GvStationList *self, GvStation *station)
 {
-	gv_station_list_insert_after(self, station, NULL);
+	gv_station_list_insert(self, station, 0);
 }
 
 void
 gv_station_list_append(GvStationList *self, GvStation *station)
 {
-	gv_station_list_insert_before(self, station, NULL);
+	gv_station_list_insert(self, station, -1);
 }
 
 void
@@ -800,18 +1042,16 @@ gv_station_list_move(GvStationList *self, GvStation *station, gint pos)
 	GvStationListPrivate *priv = self->priv;
 	GList *item;
 
+	g_return_if_fail(station != NULL);
+
 	/* Find the station */
 	item = g_list_find(priv->stations, station);
-	if (item == NULL) {
-		WARNING("GvStation %p (%s) not found in list",
-		        station, gv_station_get_uid(station));
-		return;
-	}
+	g_return_if_fail(item != NULL);
 
 	/* Move it */
+	priv->stations = g_list_insert(priv->stations, station, pos);
 	priv->stations = g_list_remove_link(priv->stations, item);
 	g_list_free(item);
-	priv->stations = g_list_insert(priv->stations, station, pos);
 
 	/* Emit a signal */
 	g_signal_emit(self, signals[SIGNAL_STATION_MOVED], 0, station);
@@ -820,16 +1060,17 @@ gv_station_list_move(GvStationList *self, GvStation *station, gint pos)
 	gv_station_list_save_delayed(self);
 }
 
-/* Move a station before another.
- * If 'before' is NULL or not found, the station is inserted at the end of the list.
- */
 void
 gv_station_list_move_before(GvStationList *self, GvStation *station, GvStation *before)
 {
 	GvStationListPrivate *priv = self->priv;
-	gint pos = -1;
+	gint pos;
+
+	g_return_if_fail(before != NULL);
 
 	pos = g_list_index(priv->stations, before);
+	g_return_if_fail(pos != -1);
+
 	gv_station_list_move(self, station, pos);
 }
 
@@ -837,23 +1078,27 @@ void
 gv_station_list_move_after(GvStationList *self, GvStation *station, GvStation *after)
 {
 	GvStationListPrivate *priv = self->priv;
-	gint pos = 0;
+	gint pos;
+
+	g_return_if_fail(after != NULL);
 
 	pos = g_list_index(priv->stations, after);
-	pos += 1; /* tricky but does what we want even for pos == -1 */
+	g_return_if_fail(pos != -1);
+
+	pos += 1;
 	gv_station_list_move(self, station, pos);
 }
 
 void
 gv_station_list_move_first(GvStationList *self, GvStation *station)
 {
-	gv_station_list_move_after(self, station, NULL);
+	gv_station_list_move(self, station, 0);
 }
 
 void
 gv_station_list_move_last(GvStationList *self, GvStation *station)
 {
-	gv_station_list_move_before(self, station, NULL);
+	gv_station_list_move(self, station, -1);
 }
 
 GvStation *
@@ -1010,6 +1255,22 @@ gv_station_list_last(GvStationList *self)
 	return g_list_last(stations)->data;
 }
 
+GvStation *
+gv_station_list_at(GvStationList *self, guint n)
+{
+	GList *stations = self->priv->stations;
+	GList *item;
+
+	if (stations == NULL)
+		return NULL;
+
+	item = g_list_nth(stations, n);
+	if (item == NULL)
+		return NULL;
+
+	return item->data;
+}
+
 GvStation *
 gv_station_list_find(GvStationList *self, GvStation *station)
 {
@@ -1102,38 +1363,19 @@ void
 gv_station_list_save(GvStationList *self)
 {
 	GvStationListPrivate *priv = self->priv;
+	const gchar *path = priv->save_path;
 	GError *err = NULL;
-	gchar *dirname = NULL;
-	gchar *text = NULL;
+	gboolean ret;
 
-	/* Stringify data */
-	text = print_markup(priv->stations, &err);
-	if (err)
-		goto cleanup;
-
-	/* Create directory */
-	dirname = g_path_get_dirname(priv->save_path);
-	if (g_mkdir_with_parents(dirname, S_IRWXU) != 0) {
-		WARNING("Failed to make directory '%s': %s", dirname, strerror(errno));
-		goto cleanup;
-	}
-
-	/* Write to file */
-	g_file_set_contents(priv->save_path, text, -1, &err);
-
-cleanup:
-	/* Cleanup */
-	g_free(dirname);
-	g_free(text);
-
-	/* Handle error */
-	if (err == NULL) {
-		INFO("Station list saved to '%s'", priv->save_path);
+	/* Save the station list */
+	ret = save_station_list_to_file(priv->stations, path, &err);
+	if (ret == TRUE) {
+		INFO("Station list saved to '%s'", path);
 	} else {
 		WARNING("Failed to save station list: %s", err->message);
-		gv_errorable_emit_error(GV_ERRORABLE(self), _("%s: %s"),
-		                        _("Failed to save station list"), err->message);
-
+		if (priv->finalization == FALSE)
+			gv_errorable_emit_error(GV_ERRORABLE(self), _("%s: %s"),
+			                        _("Failed to save station list"), err->message);
 		g_error_free(err);
 	}
 }
@@ -1142,8 +1384,6 @@ void
 gv_station_list_load(GvStationList *self)
 {
 	GvStationListPrivate *priv = self->priv;
-	const gchar *loaded_path = NULL;
-	guint i, n_paths;
 	GList *item;
 
 	TRACE("%p", self);
@@ -1151,50 +1391,55 @@ gv_station_list_load(GvStationList *self)
 	/* This should be called only once at startup */
 	g_assert_null(priv->stations);
 
-	/* Load from a list of paths */
-	n_paths = g_strv_length(priv->load_paths);
-	for (i = 0; i < n_paths; i++) {
-		const gchar *path = priv->load_paths[i];
-		GError *err = NULL;
-		gchar *text;
+	/* Try to load the station list, either from a user-provided path
+	 * (ie. during tests), either from the list of default locations.
+	 */
+	if (priv->load_path) {
+		GError *error = NULL;
+		const gchar *path = priv->load_path;
+		gboolean ret;
 
-		/* Check if the file exists */
-		if (!g_file_test(path, G_FILE_TEST_EXISTS))
-			continue;
-
-		/* Attempt to read file */
-		g_file_get_contents(path, &text, NULL, &err);
-		if (err) {
-			WARNING("%s", err->message);
-			g_clear_error(&err);
-			continue;
+		ret = load_station_list_from_file(path, &priv->stations, &error);
+		if (ret == FALSE) {
+			ERROR("Failed to load station list from '%s': %s",
+			       path, error->message);
+			/* Program execution stops here */
 		}
 
-		/* Attempt to parse it */
-		priv->stations = parse_markup(text, &err);
-		g_free(text);
-		if (err) {
-			WARNING("Failed to parse '%s': %s", path, err->message);
-			g_clear_error(&err);
-			continue;
-		}
+	} else if (priv->default_load_paths) {
+		gchar **paths = priv->default_load_paths;
+		guint n_paths = g_strv_length(paths);
+		guint i;
 
-		/* Success */
-		loaded_path = path;
-		break;
+		for (i = 0; i < n_paths; i++) {
+			const gchar *path = paths[i];
+			GError *err = NULL;
+			gboolean ret;
+
+			ret = load_station_list_from_file(path, &priv->stations, &err);
+			if (ret == FALSE) {
+				if (err->code != G_FILE_ERROR_NOENT)
+					WARNING("Failed to load station list from '%s': %s",
+						path, err->message);
+				g_clear_error(&err);
+				continue;
+			}
+
+			gv_station_list_set_load_path(self, path);
+			break;
+		}
 	}
 
 	/* Check if we got something */
-	if (loaded_path) {
-		INFO("Station list loaded from file '%s'", loaded_path);
+	if (priv->load_path) {
+		INFO("Station list loaded from file '%s'", priv->load_path);
 	} else {
-		GError *err = NULL;
+		gboolean ret;
 
 		INFO("No valid station list file found, using hard-coded default");
-
-		priv->stations = parse_markup(DEFAULT_STATION_LIST, &err);
-		if (err) {
-			ERROR("%s", err->message);
+		ret = load_station_list_from_string(DEFAULT_STATION_LIST, &priv->stations, NULL);
+		if (ret == FALSE) {
+			ERROR("Failed to load station list from hard-coded default");
 			/* Program execution stops here */
 		}
 	}
@@ -1227,6 +1472,15 @@ gv_station_list_new(void)
 	return g_object_new(GV_TYPE_STATION_LIST, NULL);
 }
 
+GvStationList *
+gv_station_list_new_with_paths(const gchar *load_path, const gchar *save_path)
+{
+	return g_object_new(GV_TYPE_STATION_LIST,
+			    "load-path", load_path,
+			    "save-path", save_path,
+			    NULL);
+}
+
 /*
  * GObject methods
  */
@@ -1266,43 +1520,6 @@ make_station_list_save_path(void)
 	return dir;
 }
 
-static void
-move_station_list_file(const gchar *new_file)
-{
-	const gchar *user_config_dir;
-	gchar *old_file = NULL;
-	gchar *dirname = NULL;
-
-	user_config_dir = gv_get_app_user_config_dir();
-	old_file = g_build_filename(user_config_dir, "stations", NULL);
-
-	if (!g_file_test(old_file, G_FILE_TEST_EXISTS))
-		goto cleanup;
-
-	INFO("Station list migration: '%s' > '%s'", old_file, new_file);
-
-	dirname = g_path_get_dirname(new_file);
-	if (g_mkdir_with_parents(dirname, S_IRWXU) != 0) {
-		WARNING("Failed to make directory '%s': %s",
-			dirname, strerror(errno));
-		goto cleanup;
-	}
-	g_free(dirname);
-
-	if (g_rename(old_file, new_file) != 0) {
-		WARNING("Failed to rename file '%s' to '%s': %s",
-			old_file, new_file, strerror(errno));
-		goto cleanup;
-	}
-
-	dirname = g_path_get_dirname(old_file);
-	g_rmdir(dirname);
-
- cleanup:
-	g_free(dirname);
-	g_free(old_file);
-}
-
 static void
 gv_station_list_finalize(GObject *object)
 {
@@ -1312,6 +1529,9 @@ gv_station_list_finalize(GObject *object)
 
 	TRACE("%p", object);
 
+	/* Indicate that the object is being finalized */
+	priv->finalization = TRUE;
+
 	/* Run any pending save operation */
 	if (priv->save_source_id > 0)
 		when_timeout_save_station_list(self);
@@ -1327,14 +1547,16 @@ gv_station_list_finalize(GObject *object)
 		g_object_add_weak_pointer(G_OBJECT(item->data), &(item->data));
 		g_object_unref(item->data);
 		if (item->data != NULL)
-			WARNING("Station '%s' has not been finalized !",
+			WARNING("Station '%s' has not been finalized!",
 			        gv_station_get_name_or_uri(GV_STATION(item->data)));
 	}
 	g_list_free(priv->stations);
 
 	/* Free paths */
 	g_free(priv->save_path);
-	g_strfreev(priv->load_paths);
+	g_free(priv->load_path);
+	g_free(priv->default_save_path);
+	g_strfreev(priv->default_load_paths);
 
 	/* Chain up */
 	G_OBJECT_CHAINUP_FINALIZE(gv_station_list, object);
@@ -1346,12 +1568,27 @@ gv_station_list_constructed(GObject *object)
 	GvStationList *self = GV_STATION_LIST(object);
 	GvStationListPrivate *priv = self->priv;
 
-	/* Initialize paths */
-	priv->load_paths = make_station_list_load_paths();
-	priv->save_path = make_station_list_save_path();
+	/* The test suite should always set custom load/save paths */
+	if (gv_in_test_suite()) {
+		if (priv->save_path == NULL || priv->load_path == NULL) {
+			ERROR("Test suite should always set station list paths");
+			/* Program execution stops here */
+		}
+	}
+
+	/* Initialize default paths */
+	priv->default_load_paths = make_station_list_load_paths();
+	priv->default_save_path = make_station_list_save_path();
+
+	/* Initialize save path if not set */
+	if (priv->save_path == NULL)
+		priv->save_path = g_strdup(priv->default_save_path);
+
+	/* DON'T initialize load path, this will be done just in time */
 
 	/* In version 4.1, the station file moved */
-	move_station_list_file(priv->save_path);
+	if (!gv_in_test_suite())
+		relocate_station_list_file_for_4_1(priv->save_path);
 
 	/* Chain up */
 	G_OBJECT_CHAINUP_CONSTRUCTED(gv_station_list, object);
@@ -1377,6 +1614,22 @@ gv_station_list_class_init(GvStationListClass *class)
 	object_class->finalize = gv_station_list_finalize;
 	object_class->constructed = gv_station_list_constructed;
 
+	/* Properties */
+	object_class->get_property = gv_station_list_get_property;
+	object_class->set_property = gv_station_list_set_property;
+
+	properties[PROP_LOAD_PATH] =
+	        g_param_spec_string("load-path", "Load Path", NULL, NULL,
+	                            GV_PARAM_DEFAULT_FLAGS | G_PARAM_READWRITE |
+				    G_PARAM_CONSTRUCT_ONLY);
+
+	properties[PROP_SAVE_PATH] =
+	        g_param_spec_string("save-path", "Save Path", NULL, NULL,
+	                            GV_PARAM_DEFAULT_FLAGS | G_PARAM_READWRITE |
+				    G_PARAM_CONSTRUCT_ONLY);
+
+	g_object_class_install_properties(object_class, PROP_N, properties);
+
 	/* Signals */
 	signals[SIGNAL_LOADED] =
 	        g_signal_new("loaded", G_TYPE_FROM_CLASS(class),
