@@ -1,7 +1,7 @@
 /*
  * Goodvibes Radio Player
  *
- * Copyright (C) 2015-2018 Arnaud Rebillout
+ * Copyright (C) 2015-2020 Arnaud Rebillout
  *
  * SPDX-License-Identifier: GPL-3.0-or-later
  *
@@ -40,12 +40,17 @@
  * Properties
  */
 
+#define DEFAULT_CLOSE_ACTION  GV_MAIN_WINDOW_CLOSE_QUIT
+#define DEFAULT_THEME_VARIANT GV_MAIN_WINDOW_THEME_DEFAULT
+
 enum {
 	/* Reserved */
 	PROP_0,
 	/* Properties */
 	PROP_STATUS_ICON_MODE,
+	PROP_PRIMARY_MENU,
 	PROP_NATURAL_HEIGHT,
+	PROP_CLOSE_ACTION,
 	PROP_THEME_VARIANT,
 	/* Number of properties */
 	PROP_N
@@ -63,7 +68,9 @@ struct _GvMainWindowPrivate {
 	 */
 
 	gboolean status_icon_mode;
+	GMenuModel *primary_menu;
 	gint     natural_height;
+	GvMainWindowCloseAction  close_action;
 	GvMainWindowThemeVariant theme_variant;
 
 	/*
@@ -135,7 +142,14 @@ gv_main_window_compute_natural_height(GvMainWindow *self)
 
 	display = gdk_display_get_default();
 	gdk_window = gtk_widget_get_window(GTK_WIDGET(self));
-	monitor = gdk_display_get_monitor_at_window(display, gdk_window);
+	if (gdk_window) {
+		monitor = gdk_display_get_monitor_at_window(display, gdk_window);
+	} else {
+		/* In status icon mode, the window might not be realized,
+		 * and in any case the window is tied to the panel which
+		 * lives on the primiary monitor. */
+		monitor = gdk_display_get_primary_monitor(display);
+	}
 	gdk_monitor_get_workarea(monitor, &geometry);
 	max_height = geometry.height;
 #else
@@ -149,7 +163,7 @@ gv_main_window_compute_natural_height(GvMainWindow *self)
 	 * Here comes a hacky piece of code !
 	 *
 	 * Problem: from the moment the station tree view is within a scrolled
-	 * window, the height is not handled smartly by GTK+ anymore. By default,
+	 * window, the height is not handled smartly by GTK anymore. By default,
 	 * it's ridiculously small. Then, when the number of rows in the tree view
 	 * is changed, the tree view is not resized. So if we want a smart height,
 	 * we have to do it manually.
@@ -252,6 +266,7 @@ make_info_tooltip_grid(GvStation *station, GvMetadata *metadata, guint bitrate)
 		const gchar *name = gv_station_get_name(station);
 		const gchar *uri = gv_station_get_uri(station);
 		const gchar *user_agent = gv_station_get_user_agent(station);
+		const gchar *codec = gv_station_get_codec(station);
 		guint nominal_bitrate = gv_station_get_nominal_bitrate(station);
 		GSList *stream_uris = gv_station_get_stream_uris(station);
 		GSList *item;
@@ -264,13 +279,24 @@ make_info_tooltip_grid(GvStation *station, GvMetadata *metadata, guint bitrate)
 		for (item = stream_uris; item; item = item->next) {
 			const gchar *stream_uri = item->data;
 
+			/* If the station uri is an audio stream, then get_steam_uris()
+			 * returns this very audio stream, and we're done.
+			 */
 			if (!g_strcmp0(stream_uri, uri))
 				continue;
 
-			grid_add_field(grid, n++, FALSE, "-", stream_uri);
+			/* Otherwise, I guess the station uri is a playlist, so let's
+			 * list the audio streams. The first audio stream is the one
+			 * being played, let's indicate that.
+			 */
+			if (item == stream_uris)
+				grid_add_field(grid, n++, FALSE, "▸", stream_uri);
+			else
+				grid_add_field(grid, n++, FALSE, "•", stream_uri);
 		}
 
 		grid_add_field(grid, n++, FALSE, _("User-agent"), user_agent);
+		grid_add_field(grid, n++, FALSE, _("Codec"), codec);
 
 		if (bitrate != 0 || nominal_bitrate != 0) {
 			gchar *str = g_strdup_printf("%u kb/s (real: %u kb/s)", nominal_bitrate, bitrate);
@@ -495,7 +521,7 @@ on_button_clicked(GtkButton *button, GvMainWindow *self)
 }
 
 /*
- * Station tree view signal handlers
+ * Stations tree view signal handlers
  */
 
 static gboolean
@@ -546,6 +572,53 @@ on_stations_tree_view_map_event(GtkWidget *stations_tree_view G_GNUC_UNUSED,
 	return GDK_EVENT_PROPAGATE;
 }
 
+/*
+ * Main window signal handlers (both popup and standalone)
+ */
+
+static gboolean
+on_window_delete_event(GvMainWindow *self,
+		       GdkEvent     *event G_GNUC_UNUSED,
+		       gpointer      data G_GNUC_UNUSED)
+{
+	GvMainWindowPrivate *priv = self->priv;
+	GvMainWindowCloseAction action = priv->close_action;
+
+	switch (action) {
+	case GV_MAIN_WINDOW_CLOSE_CLOSE:
+		/* We just close the window (ie. hide) */
+		gtk_widget_hide(GTK_WIDGET(self));
+		return GDK_EVENT_STOP;
+	case GV_MAIN_WINDOW_CLOSE_QUIT:
+	default:
+		/* We're quitting, and we do it our own way, meaning that we don't want
+		 * GTK to destroy the window, so let's stop the event propagation now.
+		 *
+		 * We don't even want to hide the window, as the window manager might
+		 * need to query the window height and position in order to save it.
+		 * Such queries would fail if the window was hidden.
+		 */
+		gv_core_quit();
+		return GDK_EVENT_STOP;
+	}
+}
+
+static gboolean
+on_window_key_press_event(GvMainWindow *self,
+                          GdkEventKey  *event,
+                          gpointer      data G_GNUC_UNUSED)
+{
+	GvMainWindowPrivate *priv = self->priv;
+
+	g_assert(event->type == GDK_KEY_PRESS);
+
+	/* Close window if <Esc> is pressed, only in status icon mode */
+	if ((event->keyval == GDK_KEY_Escape) && (priv->status_icon_mode == TRUE))
+		gtk_window_close(GTK_WINDOW(self));
+
+	return GDK_EVENT_PROPAGATE;
+}
+
 /*
  * Popup window signal handlers
  */
@@ -579,86 +652,6 @@ on_popup_window_focus_change(GtkWindow     *window,
 	return GDK_EVENT_PROPAGATE;
 }
 
-static gboolean
-on_popup_window_key_press_event(GvMainWindow *self,
-                                GdkEventKey  *event,
-                                gpointer      data G_GNUC_UNUSED)
-{
-	GvMainWindowPrivate *priv = self->priv;
-	GvPlayer *player = gv_core_player;
-
-	g_assert(event->type == GDK_KEY_PRESS);
-
-	switch (event->keyval) {
-	case GDK_KEY_Escape:
-		/* Close window if <Esc> is pressed */
-		gtk_window_close(GTK_WINDOW(self));
-		break;
-
-	case GDK_KEY_space:
-		/* Toggle playback if <Space> is pressed */
-		gtk_widget_grab_focus(priv->play_button);
-		gv_player_toggle(player);
-		return GDK_EVENT_STOP;    // consume event
-
-	default:
-		break;
-	}
-
-	return GDK_EVENT_PROPAGATE;
-}
-
-/*
- * Standalone window signal handlers
- */
-
-static gboolean
-on_standalone_window_delete_event(GvMainWindow *self G_GNUC_UNUSED,
-                                  GdkEvent     *event G_GNUC_UNUSED,
-                                  gpointer      data G_GNUC_UNUSED)
-{
-	/* We're about to quit the application */
-	gv_core_quit();
-
-	/* Do not hide the window, as there might be a save operation pending,
-	 * and hiding would make the save operation fail.
-	 */
-
-	/* We must stop the event to prevent the window from being destroyed.
-	 * The window will be destroyed later on during the cleanup process.
-	 */
-	return GDK_EVENT_STOP;
-}
-
-static gboolean
-on_standalone_window_key_press_event(GvMainWindow *self,
-                                     GdkEventKey  *event,
-                                     gpointer      data G_GNUC_UNUSED)
-{
-	GvMainWindowPrivate *priv = self->priv;
-	GvPlayer *player = gv_core_player;
-
-	g_assert(event->type == GDK_KEY_PRESS);
-
-	switch (event->keyval) {
-	case GDK_KEY_Escape:
-		/* Iconify window if <Esc> is pressed */
-		gtk_window_iconify(GTK_WINDOW(self));
-		break;
-
-	case GDK_KEY_space:
-		/* Toggle playback if <Space> is pressed */
-		gtk_widget_grab_focus(priv->play_button);
-		gv_player_toggle(player);
-		return GDK_EVENT_STOP;    // consume event
-
-	default:
-		break;
-	}
-
-	return GDK_EVENT_PROPAGATE;
-}
-
 /*
  * Construct helpers
  */
@@ -702,6 +695,16 @@ setup_setting(GtkWidget *widget, const gchar *widget_prop,
  * Public methods
  */
 
+void
+gv_main_window_play_stop(GvMainWindow *self)
+{
+	GvMainWindowPrivate *priv = self->priv;
+	GvPlayer *player = gv_core_player;
+
+	gtk_widget_grab_focus(priv->play_button);
+	gv_player_toggle(player);
+}
+
 void
 gv_main_window_resize_height(GvMainWindow *self, gint height)
 {
@@ -714,11 +717,13 @@ gv_main_window_resize_height(GvMainWindow *self, gint height)
 	gtk_window_resize(window, width, height);
 }
 
-GtkWidget *
-gv_main_window_new(GApplication *application, gboolean status_icon_mode)
+GvMainWindow *
+gv_main_window_new(GApplication *application, GMenuModel *primary_menu,
+		   gboolean status_icon_mode)
 {
 	return g_object_new(GV_TYPE_MAIN_WINDOW,
 	                    "application", application,
+			    "primary-menu", primary_menu,
 	                    "status-icon-mode", status_icon_mode,
 	                    NULL);
 }
@@ -737,12 +742,52 @@ gv_main_window_set_status_icon_mode(GvMainWindow *self, gboolean status_icon_mod
 	priv->status_icon_mode = status_icon_mode;
 }
 
+static void
+gv_main_window_set_primary_menu(GvMainWindow *self, GMenuModel *primary_menu)
+{
+	GvMainWindowPrivate *priv = self->priv;
+
+	/* This is a construct-only property */
+	g_assert_null(priv->primary_menu);
+	priv->primary_menu = g_object_ref_sink(primary_menu);
+}
+
+GMenuModel *
+gv_main_window_get_primary_menu(GvMainWindow *self)
+{
+	return self->priv->primary_menu;
+}
+
 gint
 gv_main_window_get_natural_height(GvMainWindow *self)
 {
 	return self->priv->natural_height;
 }
 
+GvMainWindowCloseAction
+gv_main_window_get_close_action(GvMainWindow *self)
+{
+	return self->priv->close_action;
+}
+
+void
+gv_main_window_set_close_action(GvMainWindow *self, GvMainWindowCloseAction action)
+{
+	GvMainWindowPrivate *priv = self->priv;
+
+	/* In status icon mode, this property is forced to CLOSE */
+	if (priv->status_icon_mode) {
+		DEBUG("Status icon mode: 'close-action' forced to CLOSE");
+		action = GV_MAIN_WINDOW_CLOSE_CLOSE;
+	}
+
+	if (priv->close_action == action)
+		return;
+
+	priv->close_action = action;
+	g_object_notify_by_pspec(G_OBJECT(self), properties[PROP_CLOSE_ACTION]);
+}
+
 GvMainWindowThemeVariant
 gv_main_window_get_theme_variant(GvMainWindow *self)
 {
@@ -789,9 +834,15 @@ gv_main_window_get_property(GObject    *object,
 	TRACE_GET_PROPERTY(object, property_id, value, pspec);
 
 	switch (property_id) {
+	case PROP_PRIMARY_MENU:
+		g_value_set_object(value, gv_main_window_get_primary_menu(self));
+		break;
 	case PROP_NATURAL_HEIGHT:
 		g_value_set_int(value, gv_main_window_get_natural_height(self));
 		break;
+	case PROP_CLOSE_ACTION:
+		g_value_set_enum(value, gv_main_window_get_close_action(self));
+		break;
 	case PROP_THEME_VARIANT:
 		g_value_set_enum(value, gv_main_window_get_theme_variant(self));
 		break;
@@ -815,6 +866,12 @@ gv_main_window_set_property(GObject      *object,
 	case PROP_STATUS_ICON_MODE:
 		gv_main_window_set_status_icon_mode(self, g_value_get_boolean(value));
 		break;
+	case PROP_PRIMARY_MENU:
+		gv_main_window_set_primary_menu(self, g_value_get_object(value));
+		break;
+	case PROP_CLOSE_ACTION:
+		gv_main_window_set_close_action(self, g_value_get_enum(value));
+		break;
 	case PROP_THEME_VARIANT:
 		gv_main_window_set_theme_variant(self, g_value_get_enum(value));
 		break;
@@ -931,7 +988,7 @@ gv_main_window_setup_widgets(GvMainWindow *self)
 }
 
 static void
-gv_main_window_setup_layout(GvMainWindow *self)
+gv_main_window_setup_appearance(GvMainWindow *self)
 {
 	GvMainWindowPrivate *priv = self->priv;
 
@@ -941,7 +998,7 @@ gv_main_window_setup_layout(GvMainWindow *self)
 }
 
 static void
-gv_main_window_configure_for_popup(GvMainWindow *self)
+gv_main_window_setup_for_popup(GvMainWindow *self)
 {
 	GtkApplicationWindow *application_window = GTK_APPLICATION_WINDOW(self);
 	GtkWindow *window = GTK_WINDOW(self);
@@ -968,13 +1025,8 @@ gv_main_window_configure_for_popup(GvMainWindow *self)
 	 */
 	gtk_window_set_modal(window, TRUE);
 
-	/* We want the window to be hidden instead of destroyed when closed */
-	g_signal_connect_object(window, "delete-event",
-	                        G_CALLBACK(gtk_widget_hide_on_delete), NULL, 0);
-
-	/* Handle some keys */
-	g_signal_connect_object(window, "key-press-event",
-	                        G_CALLBACK(on_popup_window_key_press_event), NULL, 0);
+	/* On close, we only want to close the window, instead of quitting */
+	gv_main_window_set_close_action(self, GV_MAIN_WINDOW_CLOSE_CLOSE);
 
 	/* Handle keyboard focus changes, so that we can hide the
 	 * window on 'focus-out-event'.
@@ -986,15 +1038,25 @@ gv_main_window_configure_for_popup(GvMainWindow *self)
 }
 
 static void
-gv_main_window_configure_for_standalone(GvMainWindow *self)
+gv_main_window_setup_for_standalone(GvMainWindow *self)
 {
-	/* We want to quit the application when the window is closed */
-	g_signal_connect_object(self, "delete-event",
-	                        G_CALLBACK(on_standalone_window_delete_event), NULL, 0);
+	GvMainWindowPrivate *priv = self->priv;
+	GtkMenuButton *menu_button;
+	GtkHeaderBar *header_bar;
 
-	/* Handle some keys */
-	g_signal_connect_object(self, "key-press-event",
-	                        G_CALLBACK(on_standalone_window_key_press_event), NULL, 0);
+	g_assert_nonnull(priv->primary_menu);
+
+	menu_button = GTK_MENU_BUTTON(gtk_menu_button_new());
+	gtk_menu_button_set_direction(menu_button, GTK_ARROW_NONE);
+	gtk_menu_button_set_menu_model(menu_button, priv->primary_menu);
+
+	header_bar = GTK_HEADER_BAR(gtk_header_bar_new());
+	gtk_header_bar_set_show_close_button(header_bar, TRUE);
+	gtk_header_bar_set_title(header_bar, g_get_application_name());
+	gtk_header_bar_pack_end(header_bar, GTK_WIDGET(menu_button));
+
+	gtk_window_set_titlebar(GTK_WINDOW(self), GTK_WIDGET(header_bar));
+	gtk_widget_show_all(GTK_WIDGET(header_bar));
 }
 
 /*
@@ -1018,6 +1080,8 @@ gv_main_window_configure(GvConfigurable *configurable)
 	g_assert(gv_ui_settings);
 	g_settings_bind(gv_ui_settings, "theme-variant",
 	                self, "theme-variant", G_SETTINGS_BIND_DEFAULT);
+	g_settings_bind(gv_ui_settings, "close-action",
+	                self, "close-action", G_SETTINGS_BIND_DEFAULT);
 }
 
 static void
@@ -1039,6 +1103,7 @@ gv_main_window_finalize(GObject *object)
 	TRACE("%p", object);
 
 	/* Free resources */
+	g_clear_object(&priv->primary_menu);
 	g_clear_object(&priv->info_tooltip_grid);
 
 	/* Chain up */
@@ -1057,19 +1122,26 @@ gv_main_window_constructed(GObject *object)
 	/* Build window */
 	gv_main_window_populate_widgets(self);
 	gv_main_window_setup_widgets(self);
-	gv_main_window_setup_layout(self);
+	gv_main_window_setup_appearance(self);
 
-	/* Configure depending on the window mode */
+	/* Additional setup depending on the window mode */
 	if (priv->status_icon_mode) {
-		DEBUG("Configuring main window for popup mode");
-		gv_main_window_configure_for_popup(self);
+		DEBUG("Setting up main window for popup mode");
+		gv_main_window_setup_for_popup(self);
 	} else {
-		DEBUG("Configuring main window for standalone mode");
-		gv_main_window_configure_for_standalone(self);
+		DEBUG("Setting up main window for standalone mode");
+		gv_main_window_setup_for_standalone(self);
 	}
 
+	/* Connect main window signal handlers */
+	g_signal_connect_object(self, "delete-event",
+	                        G_CALLBACK(on_window_delete_event), NULL, 0);
+	g_signal_connect_object(self, "key-press-event",
+	                        G_CALLBACK(on_window_key_press_event), NULL, 0);
+
 	/* Connect core signal handlers */
-	g_signal_connect_object(player, "notify", G_CALLBACK(on_player_notify), self, 0);
+	g_signal_connect_object(player, "notify",
+			        G_CALLBACK(on_player_notify), self, 0);
 
 	/* Chain up */
 	G_OBJECT_CHAINUP_CONSTRUCTED(gv_main_window, object);
@@ -1102,23 +1174,41 @@ gv_main_window_class_init(GvMainWindowClass *class)
 	properties[PROP_STATUS_ICON_MODE] =
 	        g_param_spec_boolean("status-icon-mode", "Status icon mode", NULL,
 	                             FALSE,
-	                             GV_PARAM_DEFAULT_FLAGS | G_PARAM_CONSTRUCT_ONLY |
-	                             G_PARAM_WRITABLE);
+	                             GV_PARAM_DEFAULT_FLAGS | G_PARAM_WRITABLE |
+				     G_PARAM_CONSTRUCT_ONLY);
+
+	properties[PROP_PRIMARY_MENU] =
+	        g_param_spec_object("primary-menu", "Primary menu", NULL,
+	                            G_TYPE_MENU_MODEL,
+	                            GV_PARAM_DEFAULT_FLAGS | G_PARAM_READWRITE |
+				    G_PARAM_CONSTRUCT_ONLY);
 
 	properties[PROP_NATURAL_HEIGHT] =
 	        g_param_spec_int("natural-height", "Natural height", NULL,
 	                         0, G_MAXINT, 0,
 	                         GV_PARAM_DEFAULT_FLAGS | G_PARAM_READABLE);
 
+	properties[PROP_CLOSE_ACTION] =
+	        g_param_spec_enum("close-action", "Close Action", NULL,
+	                          GV_TYPE_MAIN_WINDOW_CLOSE_ACTION,
+	                          DEFAULT_CLOSE_ACTION,
+	                          GV_PARAM_DEFAULT_FLAGS | G_PARAM_READWRITE);
+
 	properties[PROP_THEME_VARIANT] =
 	        g_param_spec_enum("theme-variant", "Theme variant", NULL,
 	                          GV_TYPE_MAIN_WINDOW_THEME_VARIANT,
-	                          GV_MAIN_WINDOW_THEME_DEFAULT,
+	                          DEFAULT_THEME_VARIANT,
 	                          GV_PARAM_DEFAULT_FLAGS | G_PARAM_READWRITE);
 
 	g_object_class_install_properties(object_class, PROP_N, properties);
 
 	/* Register transform function */
+	g_value_register_transform_func(GV_TYPE_MAIN_WINDOW_CLOSE_ACTION,
+	                                G_TYPE_STRING,
+	                                gv_value_transform_enum_string);
+	g_value_register_transform_func(G_TYPE_STRING,
+	                                GV_TYPE_MAIN_WINDOW_CLOSE_ACTION,
+	                                gv_value_transform_string_enum);
 	g_value_register_transform_func(GV_TYPE_MAIN_WINDOW_THEME_VARIANT,
 	                                G_TYPE_STRING,
 	                                gv_value_transform_enum_string);
