@@ -1,7 +1,7 @@
 /*
  * Goodvibes Radio Player
  *
- * Copyright (C) 2015-2018 Arnaud Rebillout
+ * Copyright (C) 2015-2020 Arnaud Rebillout
  *
  * SPDX-License-Identifier: GPL-3.0-or-later
  *
@@ -60,8 +60,11 @@ struct _GvPrefsWindowPrivate {
 
 	/* Top-level */
 	GtkWidget *window_vbox;
-	/* Misc */
-	GtkWidget *misc_vbox;
+	/* General */
+	GtkWidget *general_vbox;
+	GtkWidget *application_frame;
+	GtkWidget *application_grid;
+	GtkWidget *close_action_combo;
 	GtkWidget *playback_frame;
 	GtkWidget *playback_grid;
 	GtkWidget *autoplay_check;
@@ -347,8 +350,11 @@ gv_prefs_window_populate_widgets(GvPrefsWindow *self)
 	/* Top-level */
 	GTK_BUILDER_SAVE_WIDGET(builder, priv, window_vbox);
 
-	/* Misc */
-	GTK_BUILDER_SAVE_WIDGET(builder, priv, misc_vbox);
+	/* General */
+	GTK_BUILDER_SAVE_WIDGET(builder, priv, general_vbox);
+	GTK_BUILDER_SAVE_WIDGET(builder, priv, application_frame);
+	GTK_BUILDER_SAVE_WIDGET(builder, priv, application_grid);
+	GTK_BUILDER_SAVE_WIDGET(builder, priv, close_action_combo);
 	GTK_BUILDER_SAVE_WIDGET(builder, priv, playback_frame);
 	GTK_BUILDER_SAVE_WIDGET(builder, priv, playback_grid);
 	GTK_BUILDER_SAVE_WIDGET(builder, priv, autoplay_check);
@@ -420,7 +426,18 @@ gv_prefs_window_setup_widgets(GvPrefsWindow *self)
 	 * widgets tooltips (label + setting).
 	 */
 
-	/* Misc */
+	/* General */
+	if (status_icon_obj == NULL) {
+		setup_setting(_("Action when the close button is clicked."),
+		              NULL,
+		              priv->close_action_combo, "active-id",
+		              main_window_obj, "close-action",
+		              NULL, NULL);
+	} else {
+		setdown_widget(_("Setting not available in status icon mode."),
+		               priv->application_frame);
+	}
+
 	setup_setting(_("Whether to start playback automatically on startup."),
 	              NULL,
 	              priv->autoplay_check, "active",
@@ -472,7 +489,7 @@ gv_prefs_window_setup_widgets(GvPrefsWindow *self)
 	              main_window_obj, "theme-variant",
 	              NULL, NULL);
 
-	if (!status_icon_obj) {
+	if (status_icon_obj == NULL) {
 		setup_setting(_("Automatically adjust the window height when a station "
 		                "is added or removed."),
 		              NULL,
@@ -481,7 +498,7 @@ gv_prefs_window_setup_widgets(GvPrefsWindow *self)
 		              NULL, NULL);
 	} else {
 		setdown_widget(_("Setting not available in status icon mode."),
-		               priv->window_frame);
+		               priv->window_autosize_check);
 	}
 
 	setup_feature(_("Show notification when the status changes."),
@@ -500,7 +517,7 @@ gv_prefs_window_setup_widgets(GvPrefsWindow *self)
 	              priv->hotkeys_switch,
 	              priv->hotkeys_feat);
 
-	if (status_icon_obj) {
+	if (status_icon_obj != NULL) {
 		setup_setting(_("Action triggered by a middle click on the status icon."),
 		              priv->middle_click_action_label,
 		              priv->middle_click_action_combo, "active-id",
@@ -540,8 +557,9 @@ gv_prefs_window_setup_appearance(GvPrefsWindow *self)
 	             "spacing", 0,
 	             NULL);
 
-	/* Misc */
-	setup_notebook_page_appearance(priv->misc_vbox);
+	/* General */
+	setup_notebook_page_appearance(priv->general_vbox);
+	setup_section_appearance(priv->application_frame, priv->application_grid);
 	setup_section_appearance(priv->playback_frame, priv->playback_grid);
 	setup_section_appearance(priv->system_frame, priv->system_grid);
 	setup_section_appearance(priv->dbus_frame, priv->dbus_grid);
@@ -621,14 +639,23 @@ gv_prefs_window_class_init(GvPrefsWindowClass *class)
 static GtkWidget *
 make_prefs_window(GtkWindow *parent)
 {
-	GtkWidget *window;
+	GtkWidget *widget;
+	GtkWindow *window;
 
-	window = gv_prefs_window_new();
-	gtk_window_set_transient_for(GTK_WINDOW(window), parent);
-	gtk_window_set_destroy_with_parent(GTK_WINDOW(window), TRUE);
-	gtk_window_set_title(GTK_WINDOW(window), _("Preferences"));
+	widget = gv_prefs_window_new();
+	window = GTK_WINDOW(widget);
 
-	return window;
+	gtk_window_set_title(window, _("Preferences"));
+
+	if (parent) {
+		gtk_window_set_transient_for(window, parent);
+		gtk_window_set_destroy_with_parent(window, TRUE);
+		gtk_window_set_position(window, GTK_WIN_POS_CENTER_ON_PARENT);
+	} else {
+		gtk_window_set_position(window, GTK_WIN_POS_MOUSE);
+	}
+
+	return widget;
 }
 
 void
